/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

#include "framework.h"

#include "physics_engine.h"
#include "particle_engine.h"

#include "p_node.h"
#include "null_parent.h"
#include "state.h"
#include "debug.h"
#include "graphics_engine.h"
#include "light.h"
#include "resource_manager.h"
#include "camera.h"

int verbose;

void Framework::initModule()
{
  // Creating a Test Particle System
  ParticleSystem* system = new ParticleSystem(100000, PARTICLE_SPRITE);
  system->setLifeSpan(5);
  system->setConserve(.99);
  system->setRadius(4, 3, 1, 2);
  system->setColor(.5,0,0,.5, 1,1,0,1, 0,0,0,0);

  // Creating a Test Particle Emitter
  ParticleEmitter* emitter = new ParticleEmitter(Vector(0 , 1, 0), M_PI_4, 20, .1);
  emitter->setType(EMITTER_DOT  );
  emitter->setSize(0);
  emitter->setRelCoor(Vector(0,0,0));
  
  // Add the Flow from the Emitter into the System
  ParticleEngine::getInstance()->addConnection(emitter, system);
}

bool Framework::mainLoop()
{
  while(true)
    {
      // keyhandler returns false if sdl gets quit by some event
      if (!this->keyHandler())
	return false;

      // tick the scene
      float dt = this->tick();

      // Draw the scene
      this->draw(dt);
    }
}


bool Framework::draw(float dt)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
  glLoadIdentity(); // Reset the view
  
  ParticleEngine::getInstance()->draw(dt);

  camera->apply();
  
  SDL_GL_SwapBuffers(); // Swap the buffers
}


float Framework::tick()
{
  currFrame = SDL_GetTicks();
  float dt = (float)(currFrame - lastFrame) / 1000.0;
  lastFrame = currFrame;

  ParticleEngine::getInstance()->tick(dt);

  NullParent::getInstance()->update(dt);
  return dt;
}


bool Framework::keyHandler()
{
  // This is the main loop for the entire program and it will run until done==TRUE
  {
    // And poll for events
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
      switch (event.type) {
      case SDL_MOUSEMOTION:
	{
	  Vector view = camera->getTarget()->getAbsCoor() - camera->getAbsCoor();
	  Vector up = Vector(0, 1, 0);
	  up = camera->getAbsDir().apply(up);
	  Vector h = up.cross(view);
	  Vector v = h.cross(view);
	  h.normalize();
	  v.normalize();

	  if (mouseDown[1])
	    camera->setRelCoor(camera->getRelCoor()+ h * event.motion.xrel *.05 - v * event.motion.yrel * .05);
	  if (mouseDown[3])
	    camera->getTarget()->setRelCoor(camera->getTarget()->getRelCoor()+ h * event.motion.xrel *.05 - v * event.motion.yrel * .05);
	    
	}
	break;
      case SDL_MOUSEBUTTONDOWN:
	switch (event.button.button)
	  {
	  case 4:
	    PRINTF(4)("MouseWheel up\n");
	    //zoomTo *= .5;
	    break;
	  case 5:
	    PRINTF(4)("MouseWheel down\n");
	    //zoomTo *= 2.0;
	    break;
	  case 1:
	  case 2:
	  case 3:
	    mouseDown[event.button.button] = true;
	    break;
	  }
	    
	break;
      case SDL_MOUSEBUTTONUP:
	switch (event.button.button)
	  {
	  case 1:
	  case 2:
	  case 3:
	    mouseDown[event.button.button] = false;
	    break;
	  }
	break;
      case SDL_VIDEORESIZE:
	GraphicsEngine::getInstance()->resolutionChanged(&event.resize);
	break;
      case SDL_KEYDOWN:
	switch (event.key.keysym.sym)
	  {
	  case SDLK_x:
	    
	    break;
	  case SDLK_c:

	    break;
	  case SDLK_i:
	    ParticleEngine::getInstance()->debug();
	    break;
	  case SDLK_a:
	    //zoomTo /=2;
	    break;
	  case SDLK_z:
	    //zoomTo *=2;
	    break;
	  case SDLK_h:
	    this->printHelp();
	    break;
	  }
	break;
	    
        // If a quit event was recieved
      case SDL_QUIT:
	// then we're done and we'll end this program
	return false;
	break;
      default:
	break;
      }

    }

    // Get the state of the keyboard keys
    keys = SDL_GetKeyState(NULL);

    // and check if ESCAPE has been pressed. If so then quit
    if(keys[SDLK_ESCAPE]) return false;
  }
  return true;
}


Framework::Framework()
{
  this->lastFrame = 0;
  // Create a new OpenGL window with the title "Cone3D Basecode" at
  // 640x480x32, fullscreen and check for errors along the way
  GraphicsEngine::getInstance();

  LightManager::getInstance();
  glEnable(GL_TEXTURE_2D);

  // Build the font from a TGA image font.tga in the data directory
  // Hide the mouse cursor
  SDL_ShowCursor(2);

  for (int i = 0; i <MOUSE_BUTTON_COUNT; i++)
    mouseDown[i] = false;

  ResourceManager::getInstance()->setDataDir(DATA_DIRECTORY);
 

  initModule();

  camera = new Camera();

  State::getInstance()->setCamera(camera, camera->getTarget());

  camera->setAbsCoor(Vector(10, 10, 0));

}

Framework::~Framework()
{
  delete GraphicsEngine::getInstance();

}



void Framework::printHelp(void) const
{
  PRINT(0)("Help for the frameWork \n");
  PRINT(0)("----------------------");
  PRINT(0)("h - print Help\n");
  PRINT(0)("i - state Information\n");
  PRINT(0)("1,2 - increase/decrease emissionRate\n");
  PRINT(0)("3,4 - increase/decrease emission velocity\n");
  PRINT(0)("5,6 - increase/decrease spread angle\n");
  PRINT(0)("\n");
  PRINT(0)("7,8 - increase-decrease emitter-size\n");
  PRINT(0)("9 - swap emitterType\n");
  PRINT(0)("\n");


}



int main(int argc, char *argv[])
{
  verbose = 3;
  Framework* framework = new Framework();
  framework->mainLoop();

  delete framework;
  // Kill the GL & SDL screens
  // And quit
  return 0;
}
